import tkinter as tk
from tkinter import simpledialog, scrolledtext, messagebox, filedialog, Toplevel
import json
import os
import subprocess
import webbrowser
from datetime import datetime
import google.generativeai as genai

CONFIG_FILE = "config.json"
SCRIPTS_DIR = "scripts"
os.makedirs(SCRIPTS_DIR, exist_ok=True)

def save_config(data):
    with open(CONFIG_FILE, 'w') as f:
        json.dump(data, f)

def load_config():
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE) as f:
            return json.load(f)
    return {"api_key": None, "usage": 0}

def init_gemini():
    cfg = load_config()
    if not cfg.get("api_key"):
        raise Exception("API Key לא מוגדר.")
    genai.configure(api_key=cfg["api_key"])
    return genai.GenerativeModel("models/gemini-1.5-pro-latest")

def extract_code(text):
    if "```python" in text:
        return text.split("```python")[1].split("```", 1)[0].strip()
    return text.strip()

class GeminiApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Gemini - עוזר אישי")
        self.create_widgets()

    def add_button(self, parent, text, command, col):
        btn = tk.Button(parent, text=text, command=command)
        btn.grid(row=0, column=col, padx=5)

    def create_widgets(self):
        top_frame = tk.Frame(self.root)
        top_frame.pack(fill='x', padx=10, pady=5)

        self.add_button(top_frame, "🔐 הכנס API Key", self.set_api_key, 0)
        self.add_button(top_frame, "🌐 קבל API Key", lambda: webbrowser.open("https://aistudio.google.com/apikey"), 1)
        self.add_button(top_frame, "🧠 צור סקריפט", self.create_script, 2)
        self.add_button(top_frame, "✏️ ערוך סקריפט", self.edit_script, 3)
        self.add_button(top_frame, "🧪 נתח סקריפט", self.analyze_script, 4)
        self.add_button(top_frame, "▶️ הרץ", self.run_script, 5)
        self.add_button(top_frame, "📂 סקריפטים", self.list_scripts, 6)
        self.add_button(top_frame, "⚙️ המרה ל־EXE", self.convert_to_exe, 7)

        prompt_frame = tk.Frame(self.root)
        prompt_frame.pack(fill='x', padx=10, pady=5)

        tk.Label(prompt_frame, text="שאל את Gemini (Shift+Enter לרדת שורה, Enter לשליחה):").pack(anchor='w')
        self.prompt_entry = tk.Text(prompt_frame, height=4, wrap='word')
        self.prompt_entry.pack(fill='x')
        self.prompt_entry.bind("<Return>", self.on_enter_pressed)

        self.output = scrolledtext.ScrolledText(self.root, height=20)
        self.output.pack(fill='both', expand=True, padx=10, pady=10)

    def on_enter_pressed(self, event):
        if event.state & 0x0001:
            return
        self.create_script()
        return "break"

    def set_api_key(self):
        win = Toplevel(self.root)
        win.title("API Key")
        tk.Label(win, text="הזן את ה-API Key שלך ל-Gemini:").pack(pady=5)
        entry = tk.Entry(win, width=50)
        entry.pack(pady=5)

        def paste_from_clipboard():
            try:
                entry.delete(0, tk.END)
                entry.insert(0, self.root.clipboard_get())
            except:
                pass

        btn_frame = tk.Frame(win)
        btn_frame.pack(pady=2)
        tk.Button(btn_frame, text="📋 הדבק מהלוח", command=paste_from_clipboard).pack(side='left', padx=5)
        tk.Button(btn_frame, text="✅ שמור והמשך", command=lambda: self.save_api_key(entry.get(), win)).pack(side='left', padx=5)

    def save_api_key(self, key, win):
        if key:
            cfg = load_config()
            cfg["api_key"] = key
            save_config(cfg)
            messagebox.showinfo("נשמר", "המפתח נשמר בהצלחה.")
            win.destroy()

    def create_script(self):
        prompt = self.prompt_entry.get("1.0", "end").strip()
        if not prompt:
            return
        try:
            model = init_gemini()
            full_prompt = f"Create a Python script that {prompt}. Only output the code, without any explanation, and no extra output."
            response = model.generate_content(full_prompt)
            code = extract_code(response.text)
            self.output.insert('end', f"[Gemini תגובה מלאה]:\n{response.text}\n\n")
            filename = filedialog.asksaveasfilename(initialdir=SCRIPTS_DIR, defaultextension=".py", filetypes=[("Python files", "*.py")])
            if filename:
                with open(filename, 'w', encoding='utf-8') as f:
                    f.write(code)
                self.output.insert('end', f"[נשמר] הקובץ: {filename}\n\n")
                self.update_usage()
        except Exception as e:
            messagebox.showerror("שגיאה", str(e))

    def update_usage(self):
        cfg = load_config()
        cfg["usage"] = cfg.get("usage", 0) + 1
        save_config(cfg)

    def edit_script(self):
        path = filedialog.askopenfilename(initialdir=SCRIPTS_DIR, filetypes=[("Python files", "*.py")])
        if not path:
            return
        with open(path, 'r', encoding='utf-8') as f:
            original_code = f.read()
        instruction = simpledialog.askstring("עריכה", "מה ברצונך לשנות בקובץ?")
        if not instruction:
            return
        try:
            model = init_gemini()
            prompt = f"Here is a Python script:\n{original_code}\nPlease edit it according to this instruction: {instruction}. Only output the final edited code."
            response = model.generate_content(prompt)
            new_code = extract_code(response.text)
            timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
            new_path = path.replace(".py", f"_edited_{timestamp}.py")
            with open(new_path, 'w', encoding='utf-8') as f:
                f.write(new_code)
            self.output.insert('end', f"[✔] קובץ נערך ונשמר: {new_path}\n\n")
            self.update_usage()
        except Exception as e:
            messagebox.showerror("שגיאה בעריכה", str(e))

    def analyze_script(self):
        path = filedialog.askopenfilename(initialdir=SCRIPTS_DIR, filetypes=[("Python files", "*.py")])
        if not path:
            return
        with open(path, 'r', encoding='utf-8') as f:
            code = f.read()
        question = simpledialog.askstring("ניתוח סקריפט", "מה ברצונך לשאול על הסקריפט?")
        if not question:
            return
        try:
            model = init_gemini()
            prompt = f"Analyze the following Python script:\n{code}\n{question}"
            response = model.generate_content(prompt)
            self.output.insert('end', f"[🔍 ניתוח]:\n{response.text}\n\n")
            self.update_usage()
        except Exception as e:
            messagebox.showerror("שגיאה בניתוח", str(e))

    def run_script(self):
        path = filedialog.askopenfilename(initialdir=SCRIPTS_DIR, filetypes=[("Python files", "*.py")])
        if not path:
            return
        run_win = Toplevel(self.root)
        run_win.title("הרצת סקריפט")

        output_text = scrolledtext.ScrolledText(run_win, height=20)
        output_text.pack(fill='both', expand=True, padx=10, pady=10)

        input_frame = tk.Frame(run_win)
        input_frame.pack(fill='x', padx=10, pady=5)

        tk.Label(input_frame, text="קלט לסקריפט:").pack(anchor='w')
        input_entry = tk.Entry(input_frame)
        input_entry.pack(fill='x')

        def execute():
            user_input = input_entry.get()
            try:
                result = subprocess.run(["python", path], input=user_input, capture_output=True, text=True)
                output_text.insert('end', f"--- STDOUT ---\n{result.stdout}\n--- STDERR ---\n{result.stderr}\n\n")
            except Exception as e:
                output_text.insert('end', f"[שגיאה בהרצה] {e}\n\n")

        tk.Button(input_frame, text="▶️ הרץ", command=execute).pack(pady=5)

    def convert_to_exe(self):
        path = filedialog.askopenfilename(initialdir=SCRIPTS_DIR, filetypes=[("Python files", "*.py")])
        if not path:
            return

        folder = filedialog.askdirectory(title="בחר תיקייה לשמירת exe")
        if not folder:
            return

        try:
            subprocess.run(["pyinstaller", "--version"], capture_output=True, text=True)
        except:
            install = messagebox.askyesno("התקנת pyinstaller", "pyinstaller לא מותקן. האם ברצונך להתקין אותו?")
            if install:
                subprocess.run(["pip", "install", "pyinstaller"])
            else:
                return

        exe_win = Toplevel(self.root)
        exe_win.title("תהליך המרה ל־EXE")
        output_box = scrolledtext.ScrolledText(exe_win, height=15)
        output_box.pack(fill='both', expand=True, padx=10, pady=10)

        open_folder_var = tk.BooleanVar()
        open_folder_check = tk.Checkbutton(exe_win, text="📂 פתח תיקייה לאחר ההמרה", variable=open_folder_var)
        open_folder_check.pack()

        def run_pyinstaller():
            output_box.insert('end', f"מריץ pyinstaller על {path}\n")
          
            base_name = os.path.splitext(os.path.basename(path))[0]
            spec_path = os.path.join(folder, f"{base_name}.spec")
            dist_path = os.path.join(folder, "dist")
            build_path = os.path.join(folder, "build")
            cmd = f"pyinstaller --onefile --distpath \"{folder}\" --workpath \"{build_path}\" --specpath \"{folder}\" \"{path}\""
            process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
            for line in process.stdout:
                output_box.insert('end', line)
                output_box.see('end')
            output_box.insert('end', f"✔️ הסתיים. קובץ exe נשמר בתיקייה: {folder}\n")

            # מחיקת קבצים זמניים לאחר יצירת ה־EXE
            try:
                for fname in os.listdir(folder):
                    if fname.endswith('.spec') or fname == 'build' or fname == '__pycache__':
                        fpath = os.path.join(folder, fname)
                        if os.path.isdir(fpath):
                            subprocess.run(["rm", "-rf", fpath] if os.name != 'nt' else ["rmdir", "/S", "/Q", fpath], shell=True)
                        else:
                            os.remove(fpath)
            except Exception as cleanup_error:
                output_box.insert('end', f"⚠️ שגיאה במחיקת קבצים זמניים: {cleanup_error}\n")

            if open_folder_var.get():
                try:
                    if os.name == 'nt':
                        os.startfile(folder)
                    elif os.name == 'posix':
                        subprocess.run(["xdg-open", folder])
                except Exception as e:
                    messagebox.showerror("שגיאה בפתיחת תיקייה", str(e))

        tk.Button(exe_win, text="🚀 המר עכשיו", command=run_pyinstaller).pack(pady=5)

    def list_scripts(self):
        try:
            files = os.listdir(SCRIPTS_DIR)
            py_files = [f for f in files if f.endswith(".py")]
            if py_files:
                self.output.insert('end', "\n[📁 סקריפטים]:\n" + "\n".join(py_files) + "\n\n")
            else:
                self.output.insert('end', "\n[📁] אין סקריפטים בתיקייה.\n")
        except Exception as e:
            messagebox.showerror("שגיאה", str(e))

if __name__ == '__main__':
    root = tk.Tk()
    app = GeminiApp(root)
    root.mainloop()
